package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.SERVER_IS_LIVE
import ru.dbotthepony.mc.otm.block.CableBlock
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.graph.Graph6Node
import ru.dbotthepony.mc.otm.graph.GraphNodeListener
import ru.dbotthepony.mc.otm.graph.matter.IMatterGraphNode
import ru.dbotthepony.mc.otm.graph.matter.MatterNetworkGraph
import ru.dbotthepony.mc.otm.graph.storage.IStorageGraphNode
import ru.dbotthepony.mc.otm.graph.storage.StorageNetworkGraph
import ru.dbotthepony.mc.otm.registry.MBlockEntities

class MatterCableBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) :
	BlockEntity(MBlockEntities.MATTER_CABLE, p_155229_, p_155230_), IMatterGraphNode, GraphNodeListener {

	private var valid = true
	override val matterNode = Graph6Node<IMatterGraphNode>(this)
	private val resolverNode = LazyOptional.of { this }

	override fun invalidateCaps() {
		super.invalidateCaps()
		valid = false
	}

	override fun reviveCaps() {
		super.reviveCaps()
		valid = true
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (valid) {
			if (cap === MatteryCapability.MATTER_NODE)
				return resolverNode.cast()
		}

		return super.getCapability(cap, side)
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel)
			MatterNetworkGraph.discoverFull(this, matterNode)
	}

	override fun onNeighbour(node: Graph6Node<*>, direction: Direction) {
		val newState = blockState.setValue(CableBlock.MAPPING_CONNECTION_PROP[direction]!!, true)

		if (newState !== blockState && SERVER_IS_LIVE)
			level?.setBlock(blockPos, newState, Block.UPDATE_CLIENTS)
	}

	override fun onUnNeighbour(node: Graph6Node<*>, direction: Direction) {
		val newState = blockState.setValue(CableBlock.MAPPING_CONNECTION_PROP[direction]!!, false)

		if (newState !== blockState && SERVER_IS_LIVE)
			level?.setBlock(blockPos, newState, Block.UPDATE_CLIENTS)
	}

	override fun setRemoved() {
		super.setRemoved()
		matterNode.destroy(::MatterNetworkGraph)
	}
}

class StorageCableBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) :
	BlockEntity(MBlockEntities.STORAGE_CABLE, p_155229_, p_155230_), IStorageGraphNode, GraphNodeListener {

	private var valid = true
	private val resolverNode = LazyOptional.of { this }

	override fun attachComponents(to: StorageNetworkGraph) {}
	override fun removeComponents(from: StorageNetworkGraph) {}
	override val storageNode = Graph6Node<IStorageGraphNode>(this)

	override fun invalidateCaps() {
		super.invalidateCaps()
		valid = false
	}

	override fun reviveCaps() {
		super.reviveCaps()
		valid = true
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (valid) {
			if (cap === MatteryCapability.STORAGE_NODE)
				return resolverNode.cast()
		}

		return super.getCapability(cap, side)
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel)
			StorageNetworkGraph.discoverFull(this, storageNode)
	}

	override fun onNeighbour(node: Graph6Node<*>, direction: Direction) {
		val newState = blockState.setValue(CableBlock.MAPPING_CONNECTION_PROP[direction]!!, true)

		if (newState !== blockState && SERVER_IS_LIVE)
			level!!.setBlock(blockPos, newState, Block.UPDATE_CLIENTS)
	}

	override fun onUnNeighbour(node: Graph6Node<*>, direction: Direction) {
		val newState = blockState.setValue(CableBlock.MAPPING_CONNECTION_PROP[direction]!!, false)

		if (newState !== blockState && SERVER_IS_LIVE)
			level!!.setBlock(blockPos, newState, Block.UPDATE_CLIENTS)
	}

	override fun setRemoved() {
		super.setRemoved()

		val level = level!!

		storageNode.destroy {
			StorageNetworkGraph(level)
		}
	}
}

