package ru.dbotthepony.mc.otm.block.entity.tech

import net.minecraft.core.BlockPos
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.ForgeCapabilities
import net.minecraftforge.energy.IEnergyStorage
import ru.dbotthepony.mc.otm.block.entity.MatteryDeviceBlockEntity
import ru.dbotthepony.mc.otm.capability.FlowDirection
import ru.dbotthepony.mc.otm.capability.energy
import ru.dbotthepony.mc.otm.capability.energy.IMatteryEnergyStorage
import ru.dbotthepony.mc.otm.capability.energyStoredMattery
import ru.dbotthepony.mc.otm.capability.extractEnergy
import ru.dbotthepony.mc.otm.capability.matteryEnergy
import ru.dbotthepony.mc.otm.capability.maxEnergyStoredMattery
import ru.dbotthepony.mc.otm.capability.receiveEnergy
import ru.dbotthepony.mc.otm.capability.transcieveEnergy
import ru.dbotthepony.mc.otm.container.HandlerFilter
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.immutableList
import ru.dbotthepony.mc.otm.core.math.Decimal
import ru.dbotthepony.mc.otm.menu.tech.BatteryBankMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities

class BatteryBankBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) : MatteryDeviceBlockEntity(MBlockEntities.BATTERY_BANK, p_155229_, p_155230_), IMatteryEnergyStorage {
	var gaugeLevel by synchronizer.float()
		private set

	// 6 на 2
	val container: MatteryContainer = object : MatteryContainer(::setChanged, CAPACITY) {
		override fun setChanged(slot: Int, new: ItemStack, old: ItemStack) {
			super.setChanged(slot, new, old)
			batteryStatus[slot].value = new.getCapability(ForgeCapabilities.ENERGY).isPresent
			gaugeLevel = (batteryLevel / maxBatteryLevel).toFloat()
		}
	}.also(::addDroppableContainer)

	val batteryStatus = immutableList(CAPACITY) {
		synchronizer.bool(false)
	}

	val itemConfig = ConfigurableItemHandler(
		input = container.handler(HandlerFilter.Dischargeable.and(HandlerFilter.OnlyIn)),
		output = container.handler(HandlerFilter.OnlyOut),
		frontDefault = ItemHandlerMode.INPUT_OUTPUT,
		backDefault = ItemHandlerMode.INPUT_OUTPUT,
		leftDefault = ItemHandlerMode.INPUT_OUTPUT,
		rightDefault = ItemHandlerMode.INPUT_OUTPUT,
		topDefault = ItemHandlerMode.INPUT_OUTPUT,
		bottomDefault = ItemHandlerMode.INPUT_OUTPUT,
	)

	init {
		savetable(::container, INVENTORY_KEY)
	}

	private fun distributeEnergy(isReceiving: Boolean, howMuch: Decimal, simulate: Boolean): Decimal {
		if (!howMuch.isPositive)
			return Decimal.ZERO

		val energies = ArrayList<IEnergyStorage>()

		for (item in container) {
			if (!item.isEmpty) {
				val energy = item.matteryEnergy ?: item.energy

				if (energy != null) {
					energies.add(energy)
				}
			}
		}

		if (energies.isEmpty())
			return Decimal.ZERO

		if (howMuch.toInt() < energies.size || !energies.any { it is IMatteryEnergyStorage && it.transcieveEnergy(Decimal.ONE, isReceiving, true) == Decimal.ONE }) {
			// sequential
			var summ = Decimal.ZERO
			var remaining = howMuch

			for (energy in energies) {
				val diff = energy.transcieveEnergy(remaining, isReceiving, simulate)
				summ += diff
				remaining -= diff

				if (!remaining.isPositive) {
					break
				}
			}

			if (!simulate && !summ.isZero) {
				setChangedLight()
				gaugeLevel = (batteryLevel / maxBatteryLevel).toFloat()
			}

			return summ
		} else {
			val distribution = Array(energies.size) { Decimal.ZERO }
			var summ2 = Decimal.ZERO

			for ((i, energy) in energies.withIndex()) {
				val diff = energy.transcieveEnergy(howMuch, isReceiving, true)
				distribution[i] = diff
				summ2 += distribution[i]
			}

			if (summ2.isPositive) {
				for (i in distribution.indices) {
					distribution[i] = distribution[i] / summ2
				}
			} else {
				return Decimal.ZERO
			}

			var summ = Decimal.ZERO
			var remaining = howMuch

			for ((i, energy) in energies.withIndex()) {
				val diff = energy.transcieveEnergy((howMuch * distribution[i]).coerceAtMost(remaining), isReceiving, simulate)
				summ += diff
				remaining -= diff

				if (!remaining.isPositive) {
					break
				}
			}

			if (remaining.isPositive) {
				val diff = energies.first { it is IMatteryEnergyStorage }.transcieveEnergy(remaining, isReceiving, simulate)
				summ += diff
				remaining -= diff
			}

			if (!simulate && !summ.isZero) {
				setChangedLight()
				gaugeLevel = (batteryLevel / maxBatteryLevel).toFloat()
			}

			return summ
		}
	}

	override fun extractEnergy(howMuch: Decimal, simulate: Boolean): Decimal {
		return distributeEnergy(isReceiving = false, howMuch, simulate)
	}

	override fun receiveEnergy(howMuch: Decimal, simulate: Boolean): Decimal {
		return distributeEnergy(isReceiving = true, howMuch, simulate)
	}

	override val canSetBatteryLevel: Boolean
		get() = false

	override val energyFlow: FlowDirection
		get() = FlowDirection.BI_DIRECTIONAL

	override var batteryLevel: Decimal
		get() {
			var result = Decimal.ZERO

			for (i in 0 until container.containerSize) {
				val stack = container.getItem(i)

				if (!stack.isEmpty) {
					stack.energy?.let {
						result += it.energyStoredMattery
					}
				}
			}

			return result
		}
		set(value) {
			throw UnsupportedOperationException("Can't set battery value for battery bank")
		}

	override val maxBatteryLevel: Decimal
		get() {
			var result = Decimal.ZERO

			for (i in 0 until container.containerSize) {
				val stack = container.getItem(i)

				if (!stack.isEmpty) {
					stack.energy?.let {
						result += it.maxEnergyStoredMattery
					}
				}
			}

			return result
		}

	val energyConfig = ConfigurableEnergy(
		this,
		frontDefault = FlowDirection.OUTPUT,
		backDefault = FlowDirection.INPUT,
		leftDefault = FlowDirection.INPUT,
		rightDefault = FlowDirection.INPUT,
		topDefault = FlowDirection.INPUT,
		bottomDefault = FlowDirection.INPUT
	)

	init {
		energyConfig.front.automatePush = true
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return BatteryBankMenu(containerID, inventory, this)
	}

	companion object {
		const val CAPACITY = 6 * 2
	}
}
