package ru.dbotthepony.mc.otm.core.util

/**
 * Represents tick callback with condition, once
 */
interface IConditionalTickable : ITickable {
	/**
	 * Once this returns false, it should stay false.
	 *
	 * If it suddenly turns true after being false, result is undefined.
	 */
	val canTick: Boolean

	companion object {
		fun wrap(ticker: () -> Boolean): IConditionalTickable {
			return object : IConditionalTickable {
				override var canTick: Boolean = true
					private set

				override fun tick() {
					canTick = !ticker.invoke()
				}
			}
		}

		fun wrap(condition: () -> Boolean, ticker: () -> Unit): IConditionalTickable {
			return object : IConditionalTickable {
				override val canTick: Boolean get() = condition.invoke()

				override fun tick() {
					ticker.invoke()
				}
			}
		}
	}
}
