package ru.dbotthepony.mc.otm.menu.input

import ru.dbotthepony.mc.otm.block.entity.MatteryDeviceBlockEntity
import ru.dbotthepony.mc.otm.capability.FlowDirection
import ru.dbotthepony.mc.otm.core.immutableMap
import ru.dbotthepony.mc.otm.core.math.RelativeSide
import ru.dbotthepony.mc.otm.menu.MatteryMenu

/**
 * [allowPull] and [allowPush] controls whenever player is allowed to change these options
 */
class EnergyPlayerInput(val menu: MatteryMenu, val allowPull: Boolean = false, val allowPush: Boolean = false) {
	var possibleModes by menu.mSynchronizer.enum(FlowDirection::class.java)
		private set

	inner class Piece(val side: RelativeSide) {
		val pull = BooleanInputWithFeedback(menu)
		val push = BooleanInputWithFeedback(menu)
		val input = EnumInputWithFeedback<FlowDirection>(menu)

		var default by menu.mSynchronizer.enum(FlowDirection.NONE)

		init {
			pull.filter { allowPull }
			push.filter { allowPush }
		}

		fun configure(config: MatteryDeviceBlockEntity.ConfigurableEnergy<*>.Piece, parent: MatteryDeviceBlockEntity.ConfigurableEnergy<*>) {
			pull.with(config::automatePull)
			push.with(config::automatePush)
			input.withSupplier { config.energyFlow }.withConsumer { if (parent.possibleModes.isSupertype(it)) config.energyFlow = it }
		}
	}

	val pieces = immutableMap { for (side in RelativeSide.values()) put(side, Piece(side)) }

	// TODO
	val pull = BooleanInputWithFeedback(menu)

	// TODO
	val push = BooleanInputWithFeedback(menu)

	init {
		pull.filter { allowPull }
		push.filter { allowPush }
	}

	fun configure(config: MatteryDeviceBlockEntity.ConfigurableEnergy<*>) {
		possibleModes = config.possibleModes

		for ((side, v) in config.pieces) {
			pieces[side]!!.configure(v, config)
			pieces[side]!!.default = config.defaults[side]!!
		}

		pull.withSupplier { pieces.values.all { it.pull.value } }
		push.withSupplier { pieces.values.all { it.push.value } }

		pull.withConsumer { v -> pieces.values.forEach { it.pull.input.invoke(v) } }
		push.withConsumer { v -> pieces.values.forEach { it.push.input.invoke(v) } }
	}
}
